package extensions;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.Socket;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dareka.extensions.Extension2;
import dareka.extensions.ExtensionManager;
import dareka.processor.HttpRequestHeader;
import dareka.processor.StringResource;
import dareka.processor.Resource;
import dareka.processor.Processor;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSetMetaData;

public class UnlimitedMyList implements Extension2, Processor {
	public static final String UMYLIST_VERSION = "Unlimited MyList 1.7";
	public static final String LOG_PREFIX = "umylist";
	public static final String PROP_DEBUG = "uMyListDebug";
	private static final String[] PROCESSOR_SUPPORTED_METHODS = new String[] { "GET" };
	private static final Pattern PROCESSOR_SUPPORTED_PATTERN = Pattern.compile(
			"^(http://[^/]+\\.(?:nico|smile)video\\.jp)/umylist(/.*)$");
	private static ExtUtil util;
	private static Connection con;
	private Statement stmt;
	
	private void updateTable(String tableName,String colName,String type) {
		boolean e = false;
		final String pre = "updateTable()";
		try {
			ResultSet rs = stmt.executeQuery("PRAGMA table_info('"+tableName+"')");
			while( rs.next() ) {
				if( rs.getString("name").equals(colName) ) {
					e = true;
					break;
				}
			}
			rs.close();
			if( !e ) {
				debug(pre,"ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type);
				stmt.execute("ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type);
			}
		} catch( Exception exc ) {
		}
		debug(pre,colName+" "+e);
	}
	
	private void updateTable(String tableName,String colName,String type,String def) {
		boolean e = false;
		final String pre = "updateTable()";
		try {
			debug(pre,"PRAGMA table_info('"+tableName+"')");
			ResultSet rs = stmt.executeQuery("PRAGMA table_info('"+tableName+"')");
			while( rs.next() ) {
				debug(pre,rs.getString("name"));
				if( rs.getString("name").equals(colName) ) {
					e = true;
				}
			}
			rs.close();
			if( !e ) {
				info(pre,"ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type);
				stmt.execute("ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type+" default '"+def+"'");
			}
		} catch( Exception exc ) {
			debug(pre,"err:"+exc);
		}
		debug(pre,colName+" "+e);
	}

	public void registerExtensions(ExtensionManager mgr) {
		final String pre = "registerExtensions()";
		if( util == null ) {
			util = new ExtUtil(this, LOG_PREFIX, PROP_DEBUG);
		}
		/* クラスの読み込みよくわからないので誤魔化した */
		try {
			URL[] urls = new URL[1];
			
			urls[0] = new URL("jar:file:/sqlite-jdbc.jar!/");
			
			ClassLoader url = new URLClassLoader(urls);
			Class.forName("org.sqlite.JDBC");
		} catch(Exception e) {
		}
		if( con == null ) {
			try {
				con = DriverManager.getConnection("jdbc:sqlite:umylist.db");	
				stmt = con.createStatement();
				// if not existsがあったのでこっち使う
				stmt.execute("create table if not exists umylist(id INTEGER PRIMARY KEY AUTOINCREMENT,name text)");
				stmt.execute("create table if not exists umovie(id INTEGER PRIMARY KEY AUTOINCREMENT,mylistid integer,movieid text)");
				stmt.execute("create table if not exists mylist(id INTEGER PRIMARY KEY AUTOINCREMENT,mylistid integer)");
				updateTable("umovie","threadid","text","-1");
			} catch(Exception e) {
				info(pre,e+"");
			}
		}
		mgr.registerProcessor(this);      // Processorとして登録
	}
	
	public String getVersionString() {
		return UMYLIST_VERSION;
	}

	public String[] getSupportedMethods() {
		return PROCESSOR_SUPPORTED_METHODS;
	}

	public Pattern getSupportedURLAsPattern() {
		return PROCESSOR_SUPPORTED_PATTERN;
	}
	
	public String getSupportedURLAsString() {
		return null;
	}
	
	private HashMap getQueryParameter(String s) {
		HashMap<String,String> map = new HashMap<String,String>();
		String[] v = s.split("&");
		for(int i = 0; i < v.length; i++) {
			String[] w = v[i].split("=");
			if( w.length == 2 ) {
				map.put(w[0],w[1]);
			} else {
				map.put(w[0],"");
			}
		}
		return map;
	}
	
	private String replace(String str) {
		String ret = new String(str);
		String[] escape = {"&", "<", ">", "\"", "\'"};
		String[] replace = {"&amp;", "&lt;", "&gt;", "&quot;", "&#39;"};
		for ( int i=0; i < escape.length; i++ ){
			ret = ret.replace(escape[i], replace[i]);
		}
		return ret;
	}
	
	private boolean isAcceptMID(String id) {
		Pattern pat = Pattern.compile("^[a-z]{2}\\d+$");
		Matcher mt = pat.matcher(id);
		return mt.matches();
	}
	
	private boolean isAcceptTID(String id) {
		Pattern pat = Pattern.compile("^\\d+$");
		Matcher mt = pat.matcher(id);
		return mt.matches();
	}
	
	private boolean isAcceptMylistID(String id) {
		Pattern pat = Pattern.compile("^\\d+$");
		Matcher mt = pat.matcher(id);
		return mt.matches();
	}
	
	private int getMovieCount(int id) {
		return getMovieCount(id+"");
	}

	private int getMovieCount(String id) {
		try {
			PreparedStatement pstmt = con.prepareStatement("SELECT COUNT(*) FROM umovie where mylistid=?");
			pstmt.setString(1,id);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int ret = rs.getInt(1);
			rs.close();
			return ret;
		} catch(Exception e) {
			return 0;
		}
	}

	private String getMovies(int id) {
		return getMovies(id+"");
	}
	
	private String getMovies(String id) {
		try {
			String ret = "";
			String sql = "select * from umovie where mylistid=?";
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,replace(id));
			ResultSet rs = pstmt.executeQuery();
			while( rs.next() ) {
				if( !ret.equals("") ) ret += ",";
				ret += "{\"movieid\":\""+rs.getString("movieid")+"\",\"threadid\":\""+rs.getString("threadid")+"\"}";
			}
			return ret;
		} catch(Exception e) {
			return "";
		}
	}
	
	private void info(String path,String message) {
		util.info(path+" ["+message+"]");
	}
	
	private void debug(String path,String message) {
		if( System.getProperty("umylistDebug","false").equals("true") ) {
			info(path,"debug:"+message);
		}
	}

	private class uMyList {
		public String id;
		public String name;
		public uMyList(){
			this("","");
		}
		public uMyList(String id,String name){
			this.id = id;
			this.name = name;
		}
	}
	
	private class uMovie {
		public String movieId;
		public String threadId;
		public uMovie(){
			this("","");
		}
		public uMovie(String movieId,String threadId){
			this.movieId = movieId;
			this.threadId = threadId;
		}
	}
	
	private class MyList {
		public String id;
		public String myListId;
		public MyList(){
			this("","");
		}
		public MyList(String id,String myListId) {
			this.id = id;
			this.myListId = myListId;
		}
	}
	
	private ArrayList<uMyList> getuMyList() {
		final String pre = "getMyList()";
		ArrayList<uMyList> ret = new ArrayList<>();
		try {
			ResultSet rs = stmt.executeQuery("select * from umylist");
			ResultSetMetaData rsmd = rs.getMetaData();
			for(int now = 0; rs.next(); now++) {
				ret.add(new uMyList(rs.getString("id"),rs.getString("name")));
			}
			debug("getuMyList()","num="+ret.size());
			rs.close();
			return ret;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return ret;
	}
	
	private ArrayList<MyList> getMyList() {
		final String pre = "getMyList()";
		ArrayList<MyList> ret = new ArrayList<>();
		try {
			ResultSet rs = stmt.executeQuery("select * from mylist");
			while( rs.next() ) {
				ret.add(new MyList(rs.getString("id"),rs.getString("mylistid")));
			}
			rs.close();
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return ret;
	}
	
	private ArrayList<uMovie> getuMyList(int uMyListId) {
		return getuMyList(uMyListId+"");
	}
	
	private ArrayList<uMovie> getuMyList(String uMyListId) {
		final String pre = "getuMovie()";
		ArrayList<uMovie> ret = new ArrayList<>();
		String sql = "select * from umovie where mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			ResultSet rs = pstmt.executeQuery();
			while( rs.next() ) {
				ret.add(new uMovie(rs.getString("movieid"),rs.getString("threadid")));
			}
			rs.close();
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return ret;
	}
	
	private String encodeName(String name) {
		final String pre = "encodeName()";
		try {
			return (String)replace(URLDecoder.decode((String)name,"utf8"));
		} catch(UnsupportedEncodingException uee) {
			debug(pre,uee+"");
		}
		return "";
	}
	
	private boolean isExistName(String name) {
		String pre = "isExistName()";
		String sql = "SELECT COUNT(*) FROM umylist where name=?";
		try {
			String ename = encodeName(name);
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,ename);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num!=0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean isValiduMyListName(String name) {
		return !name.equals("");
	}
	
	private boolean adduMyList(String name) {
		final String pre = "adduMyList()";
		String sql = "INSERT INTO umylist (name) VALUES(?)";
		String ename = encodeName(name);
		if( !isValiduMyListName(ename) ) { return false; }
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,ename);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}

	private boolean isExistuMyList(int uMyListId) {
		return isExistuMyList(uMyListId+"");
	}
	
	private boolean isExistuMyList(String uMyListId) {
		final String pre = "isExistuMyList()";
		String sql = "SELECT count(*) FROM umylist WHERE id=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num!=0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean isExistMyList(int myListId) {
		return isExistMyList(myListId);
	}
	
	private boolean isExistMyList(String myListId) {
		String pre = "isExistMyList()";
		String sql = "SELECT COUNT(*) FROM mylist where mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,myListId);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num != 0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean addMyList(String myListId) {
		String pre = "addMyList()";
		String sql = "INSERT INTO mylist (mylistid) VALUES(?)";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,myListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean isExistuMovie(String uMyListId,String movieId,String threadId) {
		final String pre = "isExistuMovie()";
		String sql = "SELECT count(*) FROM umovie WHERE mylistid=? AND movieid=? AND threadid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.setString(2,movieId);
			pstmt.setString(3,threadId);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num!=0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean adduMovie(String uMyListId,String movieId,String threadId) {
		final String pre = "adduMovie()";
		String sql = "INSERT INTO umovie (mylistid,movieid,threadid) VALUES(?,?,?)";
		if( isExistuMovie(uMyListId,movieId,threadId) ) {
			return false;
		}
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.setString(2,movieId);
			pstmt.setString(3,threadId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean removeuMovie(String uMyListId,String movieId,String threadId) {
		final String pre = "removeuMovie()";
		String sql = "DELETE FROM umovie WHERE mylistid=? AND movieid=? AND threadid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.setString(2,movieId);
			pstmt.setString(3,threadId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean removeuMovie(String uMyListId) {
		final String pre = "removeuMovie()";
		String sql = "DELETE FROM umovie WHERE mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean removeuMyList(String uMyListId) {
		final String pre = "removeuMyList()";
		String sql = "DELETE FROM umylist WHERE id=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean renameuMyList(String uMyListId,String name) {
		final String pre = "renameuMyList()";
		String sql = "UPDATE umylist SET name=? WHERE id=?";
		String ename = encodeName(name);
		if( !isValiduMyListName(ename) ) {
			return false;
		}
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,ename);
			pstmt.setString(2,uMyListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean removeMyListById(int id) {
		return removeMyListById(id+"");
	}
	
	private boolean removeMyListById(String id) {
		final String pre = "removeMyListById()";
		String sql = "DELETE FROM mylist WHERE id=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,id);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}

	private boolean removeMyList(int myListId) {
		return removeMyList(myListId+"");
	}
	
	private boolean removeMyList(String myListId) {
		final String pre = "removeMyList()";
		String sql = "DELETE FROM mylist WHERE mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,myListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	public Resource onRequest(HttpRequestHeader requestHeader, Socket browser)
	throws IOException {
		try {
			Matcher m = PROCESSOR_SUPPORTED_PATTERN.matcher(requestHeader.getURI());
			if (!m.matches()) {
				throw new IOException("");
			}
			if( m.groupCount() < 2 ) {
				return util.responseText("err", "text/plain; charset=UTF-8"); 
			}
			String a = m.group(1), b = m.group(2);
			String[] tmp = b.split("\\?");
			String path = tmp[0];
			HashMap get = new HashMap();
			if( tmp.length != 1 ) {
				get = getQueryParameter(tmp[1]);
			}
			StringBuilder res = new StringBuilder();
			if( path.equals("/list") ) {
				res.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
				if( get.get("v") == null ) {
					ArrayList<uMyList> umls = getuMyList();
					res.append("<mylist>");
					for(uMyList uml : umls) {
						res.append("<list>");
						res.append("<id>"+uml.id+"</id>");
						res.append("<name>"+uml.name+"</name>");
						res.append("<moviecount>"+getMovieCount(uml.id)+"</moviecount>");
						res.append("</list>");
					}
					res.append("</mylist>");
				} else {
					String mid = (String)get.get("v");
					ArrayList<uMovie> ums = getuMyList(mid);
					debug(path,"v="+mid);
					res.append("<mylist>");
					res.append("<moviecount>"+ums.size()+"</moviecount>");
					for(uMovie um : ums) {
						res.append("<list>");
						res.append("<id>"+um.movieId+"</id>");
						res.append("<threadid>"+um.threadId+"</threadid>");
						res.append("</list>");
					}
					res.append("</mylist>");
				}
				return util.responseText(res.toString(),"application/xml; charset=UTF-8");
//				return new StringResource(res.toString());
			} else if( path.equals("/madd") ) {
				if( get.get("name") == null ) {
					return util.responseText("failed", "text/plain; charset=UTF-8");
				}
				String name = (String)get.get("name");
				debug(path,"name="+name);
				// 設定されていて同じ名前のマイリストがすでに入っている
				if( !System.getProperty("umylistUniqueMyList", "false").equals("false") && isExistName(name)) {
					debug(path,"overlap");
					return util.responseText("overlap", "text/plain; charset=UTF-8");
				}
				if( !isValiduMyListName(name) ) {
					return util.responseText("invalidName", "text/plain; charset=UTF-8"); 
				}
				if( !adduMyList(name) ) {
					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
				return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
			} else if( path.equals("/add") ) {
				if( get.get("mid") != null && get.get("mylist") != null && get.get("tid") != null ) {
					String mylist = replace((String)get.get("mylist"));
					String mid = replace((String)get.get("mid"));
					String tid = replace((String)get.get("tid"));
					debug(path,"mylist="+mylist+" mid="+mid+" tid"+tid);
					if( !isAcceptMID(mid) || !isAcceptTID(tid) ) {
						return util.responseText("invalidID", "text/plain; charset=UTF-8"); 
					}
					if( isExistuMovie(mylist,mid,tid) ) {
						return util.responseText("overlap", "text/plain; charset=UTF-8"); 
					}
					if( !isExistuMyList(mylist) ) {
						return util.responseText("invalidMylist", "text/plain; charset=UTF-8");
					}
					if( !adduMovie(mylist,mid,tid) ) {
						return util.responseText("failed", "text/plain; charset=UTF-8");
					}
					return util.responseText("succeeded", "text/plain; charset=UTF-8");
				} else {
					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/remove") ) {
				if( get.get("mid") != null && get.get("mylist") != null  && get.get("tid") != null ) {
					String mylist = (String)get.get("mylist");
					String mid = (String)get.get("mid");
					String tid = (String)get.get("tid");
					debug(path,"mylist="+mylist+" mid="+mid+" tid"+tid);
					if( !removeuMovie(mylist,mid,tid) ) {
						return util.responseText("failed", "text/plain; charset=UTF-8");
					}
					return util.responseText("succeeded", "text/plain; charset=UTF-8");
				} else {
					return util.responseText("failed", "text/plain; charset=UTF-8");
				}
			} else if( path.equals("/mremove") ) {
				if( get.get("mylist") != null ) {
					String mylist = (String)get.get("mylist");
					debug(path,"mylist="+mylist);
					if( !removeuMyList(mylist) || !removeuMovie(mylist) ) {
						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
				} else {
					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/ajax") ) {
				ArrayList<uMyList> umls = getuMyList();
				for(int i = 0; i < 2; i++) {
					StringBuilder sb = new StringBuilder();
					String s = (i==0?"{":"["),e = (i==0?"}":"]");
					sb.append(s+"\n");
					boolean flg = false;
					for(uMyList uml : umls) {
						if( flg ) {
							sb.append(",\n");
						} else {
							flg = true;
						}
						String uMylistId = uml.id;
						String name = uml.name;
						sb.append((i==0?"\""+uMylistId+"\":{":"{\"id\":"+uMylistId+",")+"\"name\":\""+name+"\",\"movies\":["+getMovies(uMylistId)+"]}");
					}
					sb.append("\n"+e+";");
					res.append("uMylist"+i+" = ");
					res.append(sb);
					res.append("\n");
				}
				return util.responseText(res.toString(),"text/javascript; charset=UTF-8");
			} else if( path.equals("/rename") ) {
				if( get.get("mylist") != null && get.get("name") != null ) {
					String mylist = replace((String)get.get("mylist"));
					String name = (String)get.get("name");
					debug(path,"mylist="+mylist+" name="+name);
					if( !isValiduMyListName(name) ) {
						return util.responseText("invalidName", "text/plain; charset=UTF-8"); 
					}
					// 設定されていて同じ名前のマイリストがすでに入っている
					if( !System.getProperty("umylistUniqueMyList", "false").equals("false") && isExistName(name) ) {
						debug(path,"overlap");
						return util.responseText("overlap", "text/plain; charset=UTF-8"); 
					}
					if( !renameuMyList(mylist,name) ) {
						return util.responseText("failed","text/javascript; charset=UTF-8");
					}
					return util.responseText("succeeded","text/javascript; charset=UTF-8");
				} else {
					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/mylist/add") ) {
				if( get.get("mylist") != null ) {
					String myListId = (String)get.get("mylist");
					if( !isAcceptMylistID(myListId) ) {
						return util.responseText("invalidMylist", "text/plain; charset=UTF-8");
					}
					if( isExistMyList(myListId) ){
						return util.responseText("overlap", "text/plain; charset=UTF-8");
					}
					if( !addMyList(myListId) ) {
						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
				} else {
					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/mylist/remove") ) {
				if( get.get("id") != null ) {
					String id = (String)get.get("id");
					debug(path,"id="+id);
					if( !removeMyListById(id) ) {
						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
				} else if( get.get("mylist") != null ) {
					String myListId = (String)get.get("mylist");
					debug(path,"myListId="+myListId);
					if( !removeMyList(myListId) ) {
						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
				} else {
					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/mylist/list") ) {
				ArrayList<MyList> mls = getMyList();
				res.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
				res.append("<mylist>");
				for(MyList ml : mls) {
					res.append("<list>");
					res.append("<id>"+ml.id+"</id>");
					res.append("<mylistid>"+ml.myListId+"</mylistid>");
					res.append("</list>");
				}
				res.append("</mylist>");
//				return new StringResource(res.toString());
				return util.responseText(res.toString(),"application/xml; charset=UTF-8");
			} else if( path.equals("/mylist/ajax") ) {
				ArrayList<MyList> mls = getMyList();
				for(int i = 0; i < 2; i++) {
					StringBuilder sb = new StringBuilder();
					String s = (i==0?"{":"["),e = (i==0?"}":"]");
					sb.append(s+"\n");
					boolean flg = false;
					for(MyList ml : mls) {
						if( flg ) {
							sb.append(",\n");
						} else {
							flg = true;
						}
						String mMylistId = ml.id;
						String mylistId = ml.myListId;
						sb.append((i==0?("\""+mMylistId+"\":{"):("{\"id\":"+mMylistId+","))+"\"mylistid\":\""+mylistId+"\"}");
					}
					sb.append("\n"+e+";");
					res.append("umMylist"+i+" = ");
					res.append(sb);
					res.append("\n");
				}
				return util.responseText(res.toString(),"text/javascript; charset=UTF-8");
			}
		} catch(Exception e) {
			return util.responseText(e+"", "text/plain; charset=UTF-8"); 
		}
		return util.responseText("", "text/plain; charset=UTF-8"); 
	}
}